@@ -1,5 +1,5 @@
 import mistune
-from .renderers import GeminiRenderer, NEWLINE, PARAGRAPH_DELIM, LINK_DELIM
+from .renderers import GeminiRenderer, NEWLINE, PARAGRAPH_DELIM, LINK_DELIM, LINEBREAK
 import argparse
 import sys
 import os
@@ -26,9 +26,8 @@ def __replace_between(text, delim, new_text, n=0):
     return start + new_text + end
 
 
-
-
-def md2gemini(markdown, img_tag="[IMG]", indent="  ", ascii_table=False, frontmatter=False, jekyll=False, links="newline", plain=False):
+def md2gemini(markdown, img_tag="[IMG]", indent="  ", ascii_table=False, frontmatter=False, jekyll=False,
+              links="newline", plain=False, strip_html=False, base_url="", md_links=False):
     """Convert the provided markdown text to the gemini format.
     
     img_tag: The text added after an image link, to indicate it's an image.
@@ -46,7 +45,13 @@ def md2gemini(markdown, img_tag="[IMG]", indent="  ", ascii_table=False, frontma
     Any other value will result in links on a newline.
 
     plain: Set to True to remove special markings from output that text/gemini doesn't support,
-    like the asterisks for bold and italics.
+    like the asterisks for bold and italics, as well as inline HTML.
+
+    strip_html: Strip all inline and block HTML from Markdown.
+
+    base_url: All links starting with a slash will have this URL prepended to them.
+
+    md_links: Convert all links to local files ending in .md to end with .gmi instead.
     """
 
     # Pre processing
@@ -71,10 +76,11 @@ def md2gemini(markdown, img_tag="[IMG]", indent="  ", ascii_table=False, frontma
         # Turn it back into text
         if md_lines != []:
             # If md_lines was empty, then effectively no removal would occur, and frontmatter would be processed
-            markdown = "\n".join(md_lines)
+            markdown = NEWLINE.join(md_lines)
     
     # Conversion
-    renderer = GeminiRenderer(img_tag=img_tag, indent=indent, ascii_table=ascii_table, links=links, plain=plain)
+    renderer = GeminiRenderer(img_tag=img_tag, indent=indent, ascii_table=ascii_table, links=links,
+                              plain=plain, strip_html=strip_html, base_url=base_url, md_links=md_links)
     gem = mistune.create_markdown(escape=False, renderer=renderer, plugins=["table", "url"])
     gemtext = gem(markdown)
     
@@ -88,6 +94,9 @@ def md2gemini(markdown, img_tag="[IMG]", indent="  ", ascii_table=False, frontma
         pg += NEWLINE * 2  # Add a blank line between paragraphs
         gemtext = __replace_between(gemtext, PARAGRAPH_DELIM, pg)
 
+    # Add in hard linebreaks
+    gemtext = gemtext.replace(LINEBREAK, NEWLINE)
+
     # Process footnotes if at-end was used
     if links == "at-end":
         gemtext += NEWLINE + renderer._render_footnotes() + NEWLINE
@@ -115,6 +124,7 @@ def md2gemini(markdown, img_tag="[IMG]", indent="  ", ascii_table=False, frontma
             # It's a link, fix the next line by removing left whitespace
             gemlines[i+1] = gemlines[i+1].lstrip()
     gemtext = NEWLINE.join(gemlines)
+    gemtext = gemtext.rstrip()
 
     return gemtext
 
@@ -122,11 +132,15 @@ def md2gemini(markdown, img_tag="[IMG]", indent="  ", ascii_table=False, frontma
 
 def __convert_file(file, args):
     if file == sys.stdin:
-        gem = md2gemini(file.read(), img_tag=args.img_tag, indent=args.indent, ascii_table=args.ascii_table, frontmatter=args.frontmatter, jekyll=args.jekyll, links=args.links, plain=args.plain)
+        gem = md2gemini(file.read(), img_tag=args.img_tag, indent=args.indent, ascii_table=args.ascii_table,
+                        frontmatter=args.frontmatter, jekyll=args.jekyll, links=args.links, plain=args.plain,
+                        strip_html=args.strip_html, base_url=args.base_url, md_links=args.md_links)
         print(gem)
     else:
         with open(file, "r") as f:
-            gem = md2gemini(f.read(), img_tag=args.img_tag, indent=args.indent, ascii_table=args.ascii_table, frontmatter=args.frontmatter, jekyll=args.jekyll, links=args.links, plain=args.plain)
+            gem = md2gemini(f.read(), img_tag=args.img_tag, indent=args.indent, ascii_table=args.ascii_table,
+                            frontmatter=args.frontmatter, jekyll=args.jekyll, links=args.links, plain=args.plain,
+                            strip_html=args.strip_html, base_url=args.base_url, md_links=args.md_links)
         if args.write:
             newfile = os.path.splitext(os.path.basename(file))[0] + ".gmi"
             with open(os.path.join(args.dir, newfile), "w") as f:
@@ -146,7 +160,10 @@ def main():
     parser.add_argument("--img-tag", type=str, help="What text to add after image links. Defaults to '[IMG]'.\nWrite something like --img-tag='' to remove it.")
     parser.add_argument("-i", "--indent", type=str, help="The number of spaces to use for list indenting. Put 'tab' to use a tab instead.")
     parser.add_argument("-l", "--links", type=str, help="Set to 'off' to turn off links, 'paragraph' to have footnotes and the real links at the end of each paragraph, or 'at-end' to have footnotes at the end of the document. Not using this flag, or having any other value will result in regular, newline links.")
-    parser.add_argument("-p", "--plain", action="store_true", help="Remove special markings from output that text/gemini doesn't support, like the asterisks for bold and italics.")
+    parser.add_argument("-p", "--plain", action="store_true", help="Remove special markings from output that text/gemini doesn't support, like the asterisks for bold and italics, and inline HTML")
+    parser.add_argument("-s", "--strip-html", action="store_true", help="Strip all inline and block HTML from Markdown. Note that using --plain will strip inline HTML as well.")
+    parser.add_argument("-b", "--base-url", type=str, help="All links starting with a slash will have this URL prepended to them.")
+    parser.add_argument("-m", "--md-links", action="store_true", help="Convert all links to local files ending in .md to end with .gmi instead.")
     args = parser.parse_args()
 
     # Validation of command line args
@@ -180,4 +197,4 @@ def main():
 
 
 __all__ = ["GeminiRenderer", "md2gemini", "main", "NEWLINE", "__version__"]
-__version__ = "1.3.2"
+__version__ = "1.5.1"
