@@ -5,19 +5,28 @@ All the renderers that convert markdown to gemini.
 import mistune
 from .unitable import UniTable, ArraySizeError
 
-NEWLINE = "\n"  # I would like to set it to \r\n, but that causes issues for bombadillo, see #150 in that repo`
+NEWLINE = "\r\n"  # For Windows support
 PARAGRAPH_DELIM = "\x02"  # The marker for paragraph start and end, for post processing
 LINK_DELIM = "\x03"
+LINEBREAK = "\x01"  # Represents a hard linebreak that should not be changed
 
 class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be used but this isn't available
     
     #NAME = "gemini"
 
-    def __init__(self, img_tag="[IMG]", indent="  ", ascii_table=False, links="newline", plain=False):
+    def __init__(self, img_tag="[IMG]", indent="  ", ascii_table=False, links="newline", plain=False,
+                 strip_html=False, base_url="", md_links=False):
         # Disable all the HTML renderer's messing around:
         super().__init__(escape=False, allow_harmful_protocols=True)
 
+        self.md_links = md_links
+        if base_url is None:
+            base_url = ""
+        if len(base_url) > 0 and base_url[-1] == "/":
+            base_url = base_url[:-1]
+        self.base_url = base_url
         self.plain = plain
+        self.strip_html = strip_html
         self.ascii = ascii_table
         if indent is None:
             self.indent = "  "
@@ -44,6 +53,10 @@ class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be u
         # are just used to denote paragraph start and end. They were picked
         # because they will never be typed in normal text editing.
 
+        link = link.strip()
+        if link.startswith("/"):
+            link = self.base_url + link
+
         if text is None:
             return LINK_DELIM + "=> " + link.strip() + LINK_DELIM
         return LINK_DELIM + "=> " + link.strip() + " " + text.strip() + LINK_DELIM
@@ -79,11 +92,21 @@ class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be u
     def link(self, link, text=None, title=None):
         # title is ignored because it doesn't apply to Gemini
 
+        if link.startswith("#"):
+            # Invalid for Gemini
+            if text is None:
+                return link
+            return text
         if self.links == "off":
             # Don't link, just leave the text as it was written
             if text is None:
                 return link
             return text
+        
+        if link.endswith(".md") and self.md_links and "//" not in link:
+            # Relative link, and md -> gmi conversion is enabled
+            link = link[:-2] + "gmi"
+
         if self.footnotes_enabled:
             if text is None or text.strip() == "":
                 # Insert the link inline, but with a footnote too
@@ -125,13 +148,15 @@ class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be u
     
     def linebreak(self):
         #return "<LB>"
-        return NEWLINE
+        return LINEBREAK
     
     def newline(self):        
         #return "<NL>"
         return ""
     
     def inline_html(self, html):
+        if self.plain or self.strip_html:
+            return ""
         return html
 
     # Block level elements
@@ -184,12 +209,23 @@ class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be u
         start = "```" + NEWLINE
         if not info is None:
             start = "```" + info + NEWLINE
-        return start + code + "```" + NEWLINE
+        
+        if code.endswith("\n"):
+            return start + code + "```" + NEWLINE*2
+        return start + code + NEWLINE + "```" + NEWLINE*2
     
     def block_quote(self, text):
-        return "> " + text.strip()
+        """Add a quote mark to the beginning of each line."""
+
+        lines = text.replace(PARAGRAPH_DELIM, "\n").strip().splitlines()
+        ret = ""
+        for line in lines:
+            ret += "> " + line.strip() + NEWLINE
+        return ret
     
     def block_html(self, html):
+        if self.strip_html:
+            return NEWLINE
         return self.block_code(html, "html")
     
     def block_error(self, html):
@@ -210,8 +246,7 @@ class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be u
 
         if start is None:
             start = 1
-        text = text.replace("\r\n", "\n")  # Make sure all newlines are the same type
-        items = text.split("\n")
+        items = text.splitlines()
         # Remove possible empty strings
         items = [x for x in items if x != ""]
         ret_items = []
@@ -267,7 +302,7 @@ class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be u
         # The table_cell func splits each column using newlines
         try:
             self.unitable.header(
-                text.split("\n")[:-1]
+                text.split("\n")[:-1]  # \n is used to delimit cells internally
             )
         except ArraySizeError:
             #raise Exception("Malformed table")
@@ -285,7 +320,7 @@ class GeminiRenderer(mistune.HTMLRenderer):  # Actually BaseRenderer should be u
         try:
             self.unitable.add_row(
                 # The table_cell func splits each column using newlines
-                text.split("\n")[:-1]
+                text.split("\n")[:-1]  # \n is used to delimit cells internally
             )
         except ArraySizeError:
             #raise Exception("Malformed table")
